<?php
namespace PME\Controllers\Parametrizacao;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;

class TipoRetiradaController {
    protected $connection;
    protected $environment;
    
    public function __construct($connection, $environment) {
        $this->connection = $connection;
        $this->environment = $environment;
    }
	
	public function saveProdutosTpRetirada(DTO\Request $request, DTO\Response $response) {
	    try {
	        $params = $request->getParameters();
	        $nrorg = $this->environment->getNrOrg();
	        
	        $queryUpdate = "UPDATE PRODRETI SET CDPRODRETI = :P_CDPRODRETI WHERE CDTIPORETI = :P_CDTIPORETI AND CDPRODRETI = :P_OLDCDPRODRETI AND NRORG = :P_NRORG";
	        $queryInsert = "INSERT INTO PRODRETI (CDTIPORETI, CDPRODRETI, NRORG) VALUES (:P_CDTIPORETI, :P_CDPRODRETI, :P_NRORG)";
	        
	        $this->connection->beginTransaction();
	        
	        foreach ($params["row"] as $data) {
	        	$params = array(
                    "P_CDTIPORETI"     => $data["CDTIPORETI"],
                    "P_CDPRODRETI"     => $data["CDPRODRETI"],
                    "P_NRORG"          => $nrorg
                );
                
	        	if ($data["__is_new"]) {
	        		$stmt = $this->connection->prepare($queryInsert);
	        	} else {
	        		$params["P_OLDCDPRODRETI"] = $data["OLDCDPRODRETI"];
	        		$stmt = $this->connection->prepare($queryUpdate);
	        	}
	        	
	        	$stmt->execute($params);
	        };
	        
	        $this->connection->commit();
	        $response->addDataSet(new DataSet("save", array(true)));
	    } catch (\Exception $e) {
	    	$this->connection->rollback();
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	public function deleteProdutosTipoRetirada(DTO\Request $request, DTO\Response $response) {
	    try {
	        $params = $request->getParameters();
	        $nrorg = $this->environment->getNrOrg();
	        
	        $queryDelete = "DELETE FROM PRODRETI WHERE CDPRODRETI = :P_CDPRODRETI AND CDTIPORETI = :P_CDTIPORETI AND NRORG = :P_NRORG";
	        
	        $this->connection->beginTransaction();

	        foreach ($params["row"] as $data) {
	        	$params = array(
                    "P_CDTIPORETI"     => $data["CDTIPORETI"],
                    "P_CDPRODRETI"     => $data["CDPRODRETI"],
                    "P_NRORG"          => $nrorg
                );
	       
	        	$stmt = $this->connection->prepare($queryDelete);
	        	$stmt->execute($params);
	        };
	        
	        $this->connection->commit();
	        $response->addDataSet(new DataSet('response', ['true']));
	    } catch (\Exception $e) {
	    	$this->connection->rollback();
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
}