<?php
/**
 * Created by PhpStorm.
 * User: matheussilva
 * Date: 06/08/14
 * Time: 11:25
 */

namespace Folha\Logic\ValesBeneficios\CalculoValeTransporte;

use \Folha\Entities\GpeAlteescala;

use \Folha\Entities\GpeEscalatrabh;

use \Folha\Entities\GpeVinculom;

use \Folha\Entities\GpeVinculoh;

use \Folha\Entities\FpaParamsindicato;

use Folha\Repositories\VabLinhavinc;

use \Folha\Repositories\FpaCalculofolha;

use \Folha\Repositories\FpaFerias;

use \Folha\Repositories\GpeSitufunch;

use \Geral\Repositories\Estruturah;

use \Folha\Entities\GpeAltesitufunc;

use \Geral\Util\Ambiente;

use \HCM\Util\DateUtil;


class DadosVinculo {

    /**
     * @var GpeVinculom $vinculo
     */
    private $vinculo;
    /**
     * @var int $composMensal
     */
    private $composMensal;
    private $sindicato;
    private $escalaTrabalho;
    private $estruturaLegal;

    private $nrorg;
    private $competencia;
    private $iniPeriodo;

    private $afastamentos;
    private $dsr;
    private $faltas;
    private $ferias;
    private $fimPeriodo;

    private $qntDiasProxMes;
    private $feriadosProxMes;
    private $tarifas;
    private $total;
    private $totalTarifas;

    const dsr_domingo = 'DOMINGO';
    const dsr_sabdom = 'SAB/DOM';

    /**
     * @param String $dtcompetencia
     * @param GpeVinculom $vinculo
     * @param int $nrorg
     * @throws \Exception
     */
    function __construct($dtcompetencia, $vinculo, $nrorg)
    {
        $this->vinculo = $vinculo;
        $this->nrorg = $nrorg;

        if($dtcompetencia instanceof \DateTime){
            $this->competencia = $dtcompetencia->format('d/m/Y');
            $competencia = $dtcompetencia;
        }else{
            $competencia = DateUtil::getDataDeString($dtcompetencia);
            $this->competencia = $dtcompetencia;
        }
        $this->iniPeriodo = $competencia->format('d/m/Y');
        $this->fimPeriodo = $competencia->format('t/m/Y');

        $this->setEstruturaLegal();
        $this->setFaltas();
        $this->setEscalaTrab();
        $this->setAfastamentos();
        $this->setFerias();
        $this->setComposMensal();
        $this->setSindicato();
        $this->compProximoMes();

        if($this->composMensal){
            $this->total = (int)$this->composMensal - (int)$this->faltas - (int)$this->afastamentos - (int)$this->ferias;
        }elseif($this->sindicato){
            $this->total = (int)$this->sindicato - (int)$this->faltas - (int)$this->afastamentos - (int)$this->ferias;
        }else{
            $this->total = (int)$this->qntDiasProxMes - (int)$this->faltas - (int)$this->afastamentos - (int)$this->ferias;
        }

        if($this->total<0){
            $this->total = 0;
        }

        $this->setTarifa();
    }

    /**
     * Busca as faltas no período
     */
    private function setFaltas()
    {
        $nrtpmovimento = '1';
        $nreventom = "490, 495, 1040, 1043";
        $dtmescompetenc = \FPA::getDtCompetencia();
        $faltas = FpaCalculofolha::getSomaItemCalculo($this->vinculo->getNrvinculom(), $this->nrorg, $this->iniPeriodo, $this->fimPeriodo, $nreventom, $nrtpmovimento, $dtmescompetenc);
        $this->faltas = (int)$faltas[0];
    }

    /**
    * Busca os Afastamentos no periodo
    */
    private function setAfastamentos()
    {

        /**
         * @var GpeAltesitufunc $altesitfunc
         */
        $total = 0;
        $iniperiodo = $this->iniPeriodo;
        $fimperiodo = $this->fimPeriodo;

        $nrsitufuncm = GpeSitufunch::getAfastamentoIncideVale($this->competencia);
        $nrsitufuncm = implode(', ', $nrsitufuncm);

        $param = array('P_NRORG' => $this->nrorg,
                       'P_NRVINCULOM' => $this->vinculo->getNrvinculom());
        $altesitfuncs = GpeAltesitufunc::getObjectsByWhere("NRORG = :P_NRORG AND NRVINCULOM = :P_NRVINCULOM AND ((DTINISITUFUNC BETWEEN '$iniperiodo' AND '$fimperiodo') OR (DTFIMSITUFUNC IS NULL OR DTFIMSITUFUNC BETWEEN '$iniperiodo' AND '$fimperiodo')) AND NRSITUFUNCM IN ($nrsitufuncm)",$param);

        if($altesitfuncs){
            foreach ($altesitfuncs as $altesitfunc) {
                $dtInicio = $altesitfunc->getDtinisitufunc();

                if (!$altesitfunc->getDtfimsitufunc()) {
                    $dtfinal = DateUtil::getDataDeString($fimperiodo);
                } else {
                    $dtfinal = DateUtil::getDataDeString($altesitfunc->getDtfimsitufunc());
                }

                $qnt = $dtInicio->diff($dtfinal, true);
                $feriados = $this->verificaFeriado($dtInicio->format('d/m/Y'), $dtfinal->format('d/m/Y'));
                $dsr = count($this->verificaDsr($dtInicio->format('d/m/Y'), $dtfinal->format('d/m/Y'), $this->dsr));

                $total+= ($qnt->d + $qnt->m*30 + $qnt->y*365) - $feriados - $dsr;
            }
        }

        $this->afastamentos = (int)$total;
    }

    /**
     * Busca a escala de trabalho do vínculo atual
     */
    private function setEscalaTrab()
    {
        $competencia = $this->competencia;
        $iniperiodo = $this->iniPeriodo;
        $fimperiodo = $this->fimPeriodo;

        /**
         * @var GpeEscalatrabh $escalaTrab
         * @var GpeAlteescala $escalaTrabVinc
         */
        $params = array('P_NRORG' => $this->nrorg,
                        'P_NRVINCULOM' => $this->vinculo->getNrvinculom());
        $escalaTrabVinc = GpeAlteescala::getObjectByWhere("NRVINCULOM = :P_NRVINCULOM AND NRORG = :P_NRORG AND '$iniperiodo' BETWEEN DTINIESCALA AND NVL(DTFIMESCALA, '$fimperiodo')", $params);
        $this->escalaTrabalho = GpeEscalatrabh::getObjectByWhere("(NRORG = :P_NRORG OR NRORG = 0 OR NRORG IS NULL) AND NRESCALATRABM = :P_NRESCALATRABM AND NRESCALATRABH IN (SELECT NRESCALATRABH FROM GPE_ESCALATRABH WHERE DTMESCOMPETENC = (SELECT MAX(MAX_ESCALATRABH.DTMESCOMPETENC) FROM GPE_ESCALATRABH MAX_ESCALATRABH WHERE MAX_ESCALATRABH.DTMESCOMPETENC <= '$competencia' AND MAX_ESCALATRABH.NRESCALATRABM = GPE_ESCALATRABH.NRESCALATRABM AND MAX_ESCALATRABH.NRORG = GPE_ESCALATRABH.NRORG))", array('P_NRESCALATRABM' => $escalaTrabVinc->getNrescalatrabm(),'P_NRORG' => $this->nrorg));

        $this->dsr = $this->escalaTrabalho->getDescansosemanal();

    }

    /**
     * Verifica quantos descansos remunerados há no periodo passado
     * @param string $inicio
     * @param string $final
     * @param string $dsr
     * @return array|int
     */
    private function verificaDsr( $inicio, $final, $dsr)
    {
        $dtInicio = DateUtil::getDataDeString($inicio);
        $dtfinal = DateUtil::getDataDeString($final);

        $trabalho = array();
        $descansos = array();

        if($dsr == self::DSR_DOMINGO){
            while($dtfinal >= $dtInicio){
                if($dtInicio->format('N') == 7){
                    $descansos[] = $dtInicio;
                }else{
                    $trabalho[] = $dtInicio;
                }
                $dtInicio->add(new \DateInterval('P01D'));
            }
        }elseif(self::DSR_SABDOM){
            while($dtfinal >= $dtInicio){
                if(($dtInicio->format('N') == 7) or ($dtInicio->format('N') == 6)){
                    $descansos[] = $dtInicio;
                }else{
                    $trabalho[] = $dtInicio;
                }
                $dtInicio->add(new \DateInterval('P01D'));
            }
        }else{
            $descansos = 0;
        }

        return $descansos;
    }


    /**
     * @param \$inicio
     * @param $final
     * @return int
     */
    private function verificaFeriado($inicio, $final)
    {
        $feriados = Estruturah::getFeriados($inicio, $final, $this->estruturaLegal, $this->competencia, $this->nrorg);
        return count($feriados);
    }

    private function setFerias()
    {
        $ferias = FpaFerias::getPeriodoGozoFerias($this->vinculo->getNrvinculom(), DateUtil::getDataDeString($this->competencia), $this->nrorg);
        if ($ferias) {
            $this->ferias = (int)$ferias['QUANTIDADE'];
        } else {
            $this->ferias = 0;
        }
    }

    private function setComposMensal()
    {
        $this->composMensal = (int) $this->vinculo->getNrdiasapurvales();
    }

    private function setSindicato()
    {
        /**
         * @var GpeVinculoh $vincHistor
         * @var FpaParamsindicato $sindicato
         */
        $dtmescompetencia = $this->competencia;

        $params = array(
            'P_NRORG' => $this->nrorg,
            'P_NRVINCULOM' => $this->vinculo->getNrvinculom());
        $vincHistor = GpeVinculoh::getObjectByWhere("NRVINCULOH = (SELECT NRVINCULOH FROM GPE_VINCULOH WHERE NRORG = :P_NRORG AND NRVINCULOM = :P_NRVINCULOM AND DTMESCOMPETENC = (SELECT MAX(MAX_VINCULOH.DTMESCOMPETENC) FROM GPE_VINCULOH MAX_VINCULOH WHERE GPE_VINCULOH.DTMESCOMPETENC <= '$dtmescompetencia' AND MAX_VINCULOH.NRVINCULOM = GPE_VINCULOH.NRVINCULOM AND MAX_VINCULOH.NRORG = GPE_VINCULOH.NRORG))", $params);

        if ($vincHistor->getNrestrutsind()) {
            $params = array('P_NRESTRUTURAM' => $vincHistor->getNrestrutsind(),
                'P_NRORG' => $this->nrorg);
            $sindicato = FpaParamsindicato::getObjectByWhere("NRESTRUTURAM = :P_NRESTRUTURAM AND NRORG = :P_NRORG AND DTINICIOVIGENCIA <= '$dtmescompetencia' AND (DTFIMVIGENCIA >= '$dtmescompetencia' OR DTFIMVIGENCIA IS NULL)", $params);
            if($sindicato)
                $this->sindicato = (int)$sindicato->getNrdiasapurvales();
            else
                $this->sindicato = 0;
        }

    }

    private function setEstruturaLegal()
    {
        /**
         * @var GpeVinculoh $vincHistor
         * @var FpaParamsindicato $sindicato
         */
        $dtmescompetencia = $this->competencia;

        $params = array(
            'P_NRORG' => $this->nrorg,
            'P_NRVINCULOM' => $this->vinculo->getNrvinculom());
        $vincHistor = GpeVinculoh::getObjectByWhere("NRVINCULOH = (SELECT NRVINCULOH FROM GPE_VINCULOH WHERE NRORG = :P_NRORG AND NRVINCULOM = :P_NRVINCULOM AND DTMESCOMPETENC = (SELECT MAX(MAX_VINCULOH.DTMESCOMPETENC) FROM GPE_VINCULOH MAX_VINCULOH WHERE GPE_VINCULOH.DTMESCOMPETENC <= '$dtmescompetencia' AND MAX_VINCULOH.NRVINCULOM = GPE_VINCULOH.NRVINCULOM AND MAX_VINCULOH.NRORG = GPE_VINCULOH.NRORG))", $params);
        $this->estruturaLegal = $vincHistor->getNrestrutlegal();

    }

    private function compProximoMes()
    {
        $competencia = DateUtil::getDataDeString($this->competencia);
        $dtinicio = DateUtil::getDataDeString($competencia->format('d/m/Y'));
        $dtfinal = DateUtil::getDataDeString($competencia->format('t/m/Y'));
        $dias = $dtinicio->diff($dtfinal, true);

        $this->feriadosProxMes = $this->verificaFeriado($dtinicio->format('d/m/Y'), $dtfinal->format('d/m/Y'));
        $dsr = count($this->verificaDsr($dtinicio->format('d/m/Y'), $dtfinal->format('d/m/Y'), $this->dsr));

        $this->qntDiasProxMes = $dias->d - $dsr - $this->feriadosProxMes;
    }

    public function getTotal(){
        return $this->total;
    }

    public function getValeTransporte()
    {
        return $this->tarifas;
    }

    public function getTotalTarifas()
    {
        return array('VALOR' => $this->totalTarifas,
                     'REFERENCIA' => $this->getTotal());
    }

    /**
     *CALCULA AS TARIFAS DO VÍNCULO
     */
    private function setTarifa()
    {
        $transps = VabLinhavinc::infoLinhaVinculoTarifa($this->vinculo->getNrvinculom(), $this->nrorg, $this->competencia);

        if ($transps) {
            $tarifa = array();
            foreach ($transps as $transp) {
                $tarifa[] = array('NRLINHATRANSP' => $transp['NRLINHATRANSP'],
                    'NRTARIFA' => $transp['NRTARIFA'],
                    'NRVINCULOM' => $transp['NRVINCULOM'],
                    'NRSENTIDOLINHA' => $transp['NRSENTIDOLINHA'],
                    'NRTIPOLINHAVINC' => $transp['NRTIPOLINHAVINC'],
                    'QTLINHATARIFA' => $transp['QTLINHATARIFA'],
                    'VALOR' => $transp['VRTARIFA'] * $transp['QTLINHATARIFA'] * $this->total);
                $this->totalTarifas += $transp['VRTARIFA'] * $transp['QTLINHATARIFA'] * $this->total;
            }
        }else{
            $tarifa = 0;
        }
        $this->tarifas = $tarifa;
    }



} 